/*	exs.h	*/

#ifndef EXS_H_
#define EXS_H_

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012, 2013 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grant No. OCI-1127228.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */


/* these includes are necessary to compile the code in this file by itself */
#include <sys/time.h>
#include <limits.h>
#include <poll.h>

#ifdef __cplusplus
extern "C" {
#endif

#define	EXS_VERSION		1	/* Current implementation version */
#define EXS_SUBVERSION		3	/* Current implementation subversion */
#define EXS_SUBSUBVERSION	2	/* Current subsubversion */

typedef void* exs_ahandle_t;	/* Asynchronous Handle. */

/* constants for flags parameter to exs_recv() and exs_send() */
/* these should be compatible with the MSG_XXXX symbols in <bits/socket.h> */
#define EXS_DONTWAIT			(0x000040)	/* i.e., MSG_DONTWAIT */
#define EXS_CREDIT_WAIT			(0x080000)
#define EXS_BLOCK			(0x040000)
#define EXS_UNSIGNALED			(0x020000)

/* constants for flags parameter to exs_mregister() */
#define EXS_LOCAL_READ		0x01
#define EXS_LOCAL_WRITE		0x02
#define EXS_REMOTE_READ		0x04
#define EXS_REMOTE_WRITE	0x08
#define EXS_ACCESS_ALL		0x0F
#define	EXS_MRF_SHARED		0x10
#define EXS_ALL_FLAGS		0x1F

typedef struct exs_mhandle *	exs_mhandle_t;	/* Registered Memory Handle */
#define EXS_MHANDLE_INVALID	 ((exs_mhandle_t)1)
#define EXS_MHANDLE_UNREGISTERED ((exs_mhandle_t)2)

typedef struct no_such_struct *	exs_qhandle_t;	/* User's Event Queue Handle */
#define EXS_QHANDLE_INVALID	((exs_qhandle_t)-1L)

#define	EXS_QATTR_DEPTH		1
#define	EXS_QATTR_SIGNAL	2
#define	EXS_QATTR_EVENTS	3

typedef struct exs_signal exs_signal_t;
#define	EXS_SIG_ENABLE	1
#define	EXS_SIG_DISABLE	2

typedef struct exs_pollfd exs_pollfd_t;

#ifdef POLLIN
	#define	EXS_POLLIN	POLLIN		/* 0x001 There is data to read. */
#else
	#define EXS_POLLIN	0x01
#endif

#ifdef POLLOUT
	#define EXS_POLLOUT	POLLOUT		/*0x004 Writing now will not block*/
#else
	#define	EXS_POLLOUT	0x04
#endif

typedef struct exs_acceptaddr exs_acceptaddr_t;

typedef struct exs_iovec exs_iovec_t;

typedef struct exs_msghdr exs_msghdr_t;

typedef struct exs_fdvec exs_fdvec_t;

typedef struct exs_xferfile exs_xferfile_t;

#define EXS_IOVEC 1
#define EXS_FDVEC 2

#define EXS_SHUT_WR 1

#define SO_ASYNC_RECV_ORDERED 35
#define SO_ASYNC_SEND_TIMEOUT 36
#define SO_ASYNC_RECV_TIMEOUT 37

#define EXS_TIMEOUT_ONCE 1

typedef struct exs_event exs_event_t;
#define	EXS_EVT_POLL		1
#define	EXS_EVT_CONNECT		2
#define	EXS_EVT_ACCEPT		3
#define	EXS_EVT_SEND		4
#define	EXS_EVT_RECV		5
#define	EXS_EVT_SENDMSG		6
#define EXS_EVT_RECVMSG		7
#define	EXS_EVT_SENDFILE	8
#define	EXS_EVT_SENDTIMEOUT	9
#define	EXS_EVT_RECVTIMEOUT	10
#define EXS_EVT_CLOSE		11	/* added by us */
#define EXS_EVT_SHUTDOWN	12	/* added by us */


/* command values added by us for use in exs_fcntl()
 * as values of the 2nd parameter
 */
#define EXS_F_GETSPMAXSIZE		4096
#define EXS_F_SETSPMAXSIZE		4097
#define EXS_F_GETFD			4102
#define EXS_F_SETFD			4103
#define EXS_F_SETCOMPTHREADCPU		4104
#define EXS_F_GETFLOWCONTROLCREDITS	4105
#define EXS_F_SETFLOWCONTROLCREDITS	4106
#define EXS_F_GETINLINEMAXSIZE		4107
#define EXS_F_SETINLINEMAXSIZE		4108
#define EXS_F_GETSTREAMBUFSIZE		4109
#define EXS_F_SETSTREAMBUFSIZE		4110
#define EXS_F_GETCOMPTHREADCPU		4111

/* flag values for use as 3rd parameter in exs_fcntl() with GETFD/SETFD */
#define EXS_FD_SP_COPY_UNREG_ON_SEND	0x0001
#define EXS_FD_BYPASS_PARAM_CHECKING	0x0002
#define EXS_FD_NODELAY			0x0004	/* TCP_NODELAY equivalent */
#define EXS_FD_BUSYPOLL			0x0008  /* busy poll RDMA completions */
#define EXS_FD_SP_COPY_UNREG_ON_RECV	0x0010
#define EXS_FD_SP_COPY_UNREG		0x0011  /* alias for both bits */

/* below exs_fcntl() GETFD/SETFD flags are for debugging purposes only */
/* force iWARP support mode (i.e., do not use RDMA_WRITE_WITH_IMM) */
#define EXS_FD_NO_WRITE_WITH_IMM        0x0020


/* RDMA tuning flags for exs_getsockopt() and exs_setsockopt().  This will be
 * kept in sync with the rsocket project for all tuning parameters that make
 * sense in UNH EXS. */
#define SOL_RDMA 0x10000
enum sockopt_rdma {
	RDMA_SQSIZE,
	RDMA_RQSIZE,
	RDMA_INLINE,
	RDMA_IOMAPSIZE
};


/* This limit is specified by the ES-API, but we don't use it since we do not
 * enforce such a limit.  Note that you should not attempt to allocate an array
 * based on this value. */
#define EXS_EVTVEC_MAX INT_MAX

typedef struct exs_evt_poll exs_evt_poll_t;

typedef struct exs_evt_accept exs_evt_accept_t;

typedef struct exs_evt_xfer exs_evt_xfer_t;

typedef struct exs_evt_xfermsg exs_evt_xfermsg_t;

typedef struct exs_evt_xferfile exs_evt_xferfile_t;

#define EXS_CAF_FILDES 1
#define EXS_CAF_AHANDLE 2

struct exs_signal
{
	int exs_sigstate;
	int exs_signo;
};

struct exs_pollfd
{
	int		exs_fildes;	/* File descriptor. */
	int		exs_events;	/* Input event flags. */
	exs_ahandle_t	exs_ahandle;	/* Asynchronous handle. */
};

struct exs_acceptaddr
{
	struct sockaddr	*exs_addr;	/* Optional address. */
	socklen_t	exs_addrlen;	/* Size of address. */
	exs_ahandle_t	exs_ahandle;
};

struct exs_iovec
{
	void		*iov_base;	/* Base addressof a memory region. */
	size_t		iov_len;	/* Size of memory pointed to
					   by iov_base. */
	exs_mhandle_t	iov_mhandle;	/* mhandle tied to memory at iov_base.*/
};

struct exs_msghdr
{
	void		*msg_name;	/* Optional address. */
	socklen_t	msg_namelen;	/* Size of address. */
	exs_iovec_t	*msg_iov;	/* Scatter/gather array. */
	int		msg_iovlen;	/* Members in exs_msg_iov. */
	void		*msg_control;	/* Acilliary data. */
	socklen_t	mgs_controllen;	/* Ancilliary data buffer length. */
	int		msg_flags;	/* Flags on received message. */
};

struct exs_fdvec
{
	int		exs_fildes;	/* File descriptor. */
	off_t		exs_offset;	/* Offset into the file. */
	size_t		exs_length;	/* Requested transfer length. */
	int		exs_flags;	/* Flags. */
	void		*exs_location;	/* Location returned by mmap() */
};

union exs_xfvec
{
	exs_iovec_t	exs_iov;	/* IOVEC extent. */
	exs_fdvec_t	exs_fdv;	/* FDVEC extent. */
};

struct exs_xferfile
{
	int		exs_xfv_type;	/* Source extent type. */
	union exs_xfvec	exs_xfv;	/* Source extent. */
};

struct exs_timeout_t
{
	struct timeval	exs_duration;	/* Timeout value. */
	int		exs_flags;	/* Flags specifying timer
					   restart behavior. */
	exs_qhandle_t	exs_qhandle;	/* Queue to which the events
					   will be delivered. */
	exs_ahandle_t	exs_ahandle;	/* Application supplied handle. */
};

struct exs_evt_poll
{
	int		exs_evt_events;
};

struct exs_evt_accept
{
	int		exs_evt_new_socket;
	struct sockaddr	*exs_evt_addr;
	socklen_t	exs_evt_addrlen;
};

struct exs_evt_xfer
{
	size_t		exs_evt_length;	/* Number of bytes transferred. */
	void		*exs_evt_buffer;
	exs_mhandle_t	exs_evt_mhandle;
	/* next field should be non-zero only if truncated a SOCK_SEQPACKET */
	size_t		exs_evt_amount_lost;
	/* the next 2 fields are used only for exs_recv */
	size_t		exs_evt_buffer_length;
	void		*__transaction;  /* user should not see this */
};

struct exs_evt_xfermsg
{
	size_t		exs_evt_length;	/* Number of bytes transferred. */
	exs_msghdr_t	*exs_evt_msg;
};

struct exs_evt_xferfile
{
	size_t		exs_evt_length;	/* Number of bytes transferred. */
	exs_xferfile_t	*exs_evt_sendvec;
	int		exs_evt_sendvec_cnt;
};

union exs_evt_u
{
	exs_evt_poll_t		exs_evt_poll;
	exs_evt_accept_t	exs_evt_accept;
	exs_evt_xfer_t		exs_evt_xfer;
	exs_evt_xfermsg_t	exs_evt_xfermsg;
	exs_evt_xferfile_t	exs_evt_xferfile;
};

struct exs_event
{
	int			exs_evt_type;
	int			exs_evt_errno;	/* Status: 0 means success. */
	exs_ahandle_t		exs_evt_ahandle;/* Set to value passed
						   by application. */
	int			exs_evt_socket;
	struct	{				/* struct our_list_head */
		void *next;
		void *prev;
		}		queue_list;	/* for threading onto queues */
	union exs_evt_u		exs_evt_union;	/*Union event-specific structs*/
};

/* Function Forward Declarations */
int		exs_init (int);
exs_mhandle_t	exs_mregister (void*, size_t, int);
int		exs_mmodify (exs_mhandle_t, size_t, int);
int		exs_mderegister (exs_mhandle_t, int);
exs_qhandle_t	exs_qcreate (int);
int		exs_qdelete (exs_qhandle_t);
int		exs_qmodify (exs_qhandle_t, int, void*, size_t);
int		exs_qdequeue (exs_qhandle_t, exs_event_t*, int,
			const struct timeval*);
int		exs_qstatus (exs_qhandle_t, int, void*, size_t);
int		exs_cancel (int, int, exs_ahandle_t);
int		exs_accept (int, exs_acceptaddr_t*, int, int,
			exs_qhandle_t);
int		exs_blocking_accept (int, struct sockaddr *, socklen_t *);
int		exs_connect (int, const struct sockaddr*, socklen_t,
			int, struct timeval*, exs_qhandle_t, exs_ahandle_t);
int		exs_blocking_connect (int, const struct sockaddr *, socklen_t);
ssize_t		exs_send (int, const void *, size_t, int, exs_qhandle_t,
			exs_ahandle_t, exs_mhandle_t);
ssize_t		exs_blocking_send (int, void *, size_t, int, exs_mhandle_t);
ssize_t		exs_sendmsg (int, const exs_msghdr_t*, int,
			exs_qhandle_t, exs_ahandle_t);
ssize_t		exs_recv (int, void *, size_t, int, exs_qhandle_t,
			exs_ahandle_t, exs_mhandle_t);
ssize_t		exs_blocking_recv (int, void *, size_t, int, exs_mhandle_t);
ssize_t		exs_recvmsg (int, exs_msghdr_t*, int, exs_qhandle_t,
			exs_ahandle_t);
ssize_t		exs_sendfile (int, exs_xferfile_t*, int, int,
			exs_qhandle_t, exs_ahandle_t);
nfds_t		exs_poll (exs_pollfd_t*, nfds_t, int, exs_qhandle_t);

int		exs_socket (int family, int socktype, int protocol);
int		exs_bind (int fd, struct sockaddr *address, socklen_t addrlen);
int		exs_listen (int fd, int backlog);
int		exs_close (int s, int flags, exs_qhandle_t, exs_ahandle_t);
int		exs_shutdown(int s, int how, int flags, exs_qhandle_t, exs_ahandle_t);
int		exs_blocking_close (int sockno);
int		exs_getsockname (int s, struct sockaddr *name,
			socklen_t *namelen);
int		exs_getpeername (int s, struct sockaddr *name,
			socklen_t *namelen);
long		exs_fcntl (int fd, int command_name, long command_value);
const char *	exs_evt_type_to_string (int value);

int		exs_getsockopt(int sockfd, int level, int optname,
			void *optval, socklen_t *optlen);
int		exs_setsockopt(int sockfd, int level, int optname,
			const void *optval, socklen_t optlen);

void		exs_reset_counters (void);

void		exs_print_counters (FILE *);

void		exs_dump_counters(void);

void		exs_print_calloc_report (FILE *);

int		exs_print_connection_attributes(FILE *, int, char *);

int		exs_set_debug_flag (int value);

int		exs_get_version(char *buf, size_t size);

ssize_t		exs_read (int sockno, void *buffer, size_t max_bytes);

ssize_t		exs_write (int sockno, void *buffer, size_t max_bytes);

#ifdef __cplusplus
}
#endif

#endif /* EXS_H_ */

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
